// Set Goal Position By Sync
//  Move position to the full range for XM430/XH430
//  Since software serial is selected, dynamixel should be use at 57600bps or less.
//  1.select [SOFT]
//  2.compile & download

#include <dx2lib.h>
#include <SoftwareSerial.h>

#define MAXAXISNUM  2

DX2LIB dxif (true); // select software serial

int32_t maxpos[MAXAXISNUM], minpos[MAXAXISNUM], gpos[MAXAXISNUM], dir[MAXAXISNUM];

void setup() {
  dxif.begin (57600);

  // Get max/min position limit
  for (int i = 0; i < MAXAXISNUM; i++) {
    dxif.ReadLongData (1, 48, (uint32_t *)&maxpos[i], NULL);
    dxif.ReadLongData (1, 52, (uint32_t *)&minpos[i], NULL);
  }

  // Torque on
  uint8_t ids[MAXAXISNUM];
  uint8_t data[MAXAXISNUM];
  for (int i = 0; i < MAXAXISNUM; i++) {
    ids[i] = i + 1;
    data[i] = 1;
    dir[i] = 10;
  }
  dxif.WriteSyncByteData (ids, 64, data, MAXAXISNUM);

  // Get PresentPosition
  for (int i = 0; i < MAXAXISNUM; i++) dxif.ReadLongData (i + 1, 116, (uint32_t *)&gpos[i], NULL);
}

void loop() {
  uint8_t ids[MAXAXISNUM];
  uint32_t data[MAXAXISNUM];

  // Update target value
  for (int i = 0; i < MAXAXISNUM; i++) {
    gpos[i] += dir[i];
    if (gpos[i] >= maxpos[i]) {
      gpos[i] = maxpos[i];
      dir[i] *= -1;
    } else if (gpos[i] <= minpos[i]) {
      gpos[i] = minpos[i];
      dir[i] *= -1;
    }
    ids[i] = i + 1;
    data[i] = gpos[i];
  }

  // Set goal position
  dxif.WriteSyncLongData (ids, 116, (uint32_t *)&data, 2);

  delay (10);
}
